use std::env;
use std::fs::{File, Permissions};
use std::io::Write;
use std::os::unix::fs::PermissionsExt;
use std::path::Path;
use log::info;
use rand::rngs::OsRng;
use ssh_key::{PrivateKey, Algorithm, LineEnding};


/** Check if the SSH pubkey/privkey exist at the configured paths. If not, generate them. */
pub fn ensure_ssh_keypair() -> Result<(), ssh_key::Error> {
    let pubkey_path = env::var("P5X_SSH_PUBKEY_PATH").expect("Missing env: P5X_SSH_PUBKEY_PATH");
    let privkey_path = env::var("P5X_SSH_PRIVKEY_PATH").expect("Missing env: P5X_SSH_PRIVKEY_PATH");

    // If both exist, then p5x will boot correctly when it reads from the files.
    // If only one of the two files exists, p5x will error on boot. This is safer
    // than accidentally overwriting someone's key.
    if Path::new(&pubkey_path).exists() || Path::new(&privkey_path).exists() {
        info!(target: "p5x", "Found existing SSH keypair on filesystem");
        return Ok(());
    }

    // Generate an ed25519 keypair
    info!(target: "p5x", "Generating a new SSH keypair");
    let mut csprng = OsRng;
    let privkey = PrivateKey::random(&mut csprng, Algorithm::Ed25519)?;

    // Write the privkey to a file
    let privkey_pem = privkey.to_openssh(LineEnding::LF)?;
    let mut privkey_file = File::create(privkey_path)?;
    privkey_file.write_all(privkey_pem.as_bytes())?;
    privkey_file.set_permissions(Permissions::from_mode(0o600))?;

    // Write the pubkey to a file
    let pubkey = privkey.public_key();
    let pubkey_ssh = pubkey.to_openssh()?;
    let mut pubkey_file = File::create(pubkey_path)?;
    pubkey_file.write_all(pubkey_ssh.as_bytes())?;

    Ok(())
}
